# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-08-06 19:54
from __future__ import unicode_literals

from django.db import migrations
from django.db import models


class Migration(migrations.Migration):

    dependencies = [
        ('orchestra', '0066_remove_taskassignment_integrity_constraint'),
    ]

    operations = [
        migrations.AddField(
            model_name='staffbotrequest',
            name='last_inquiry_sent',
            field=models.DateTimeField(null=True),
        ),
    ]
