{% extends 'registration/registration_base.html' %}

{% block title %}Reset password{% endblock %}

{% block registration_block %}
    <input type="text" class="form-control" placeholder="E-mail"
           id="id_email" maxlength="30" name="email"
           value="{% firstof form.email.value '' %}" autofocus>
    <div class="error-field"> {{ form.email.errors.0 }} </div>
    <br>
    <div class="error-non-field"> {{ form.non_field_errors.0 }} </div>
    <button class="btn btn-block btn-primary" type="submit"><i class="fa fa-lock"></i> RESET PASSWORD</button>
    <hr>
    <div class="registration">
        <a href="{% url 'auth_login' %}">Sign in</a>
    </div>
{% endblock registration_block %}
